@@ -1,6 +1,7 @@
 /* filelist.c
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -28,6 +29,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "options.h"
 
 gib_list *filelist = NULL;
+gib_list *original_file_items = NULL; /* original file items from argv */
 int filelist_len = 0;
 gib_list *current_file = NULL;
 extern int errno;
@@ -111,6 +113,11 @@ gib_list *feh_file_remove_from_list(gib_list * list, gib_list * l)
 	return(gib_list_remove(list, l));
 }
 
+int file_selector_all(const struct dirent *unused __attribute__((unused)))
+{
+  return 1;
+}
+
 /* Recursive */
 void add_file_to_filelist_recursively(char *origpath, unsigned char level)
 {
@@ -150,33 +157,35 @@ void add_file_to_filelist_recursively(char *origpath, unsigned char level)
 
 	errno = 0;
 	if (stat(path, &st)) {
-		/* Display useful error message */
-		switch (errno) {
-		case ENOENT:
-		case ENOTDIR:
-			if (!opt.quiet)
+		if (!opt.quiet) {
+			switch (errno) {
+			case ENOENT:
+			case ENOTDIR:
 				weprintf("%s does not exist - skipping", path);
-			break;
-		case ELOOP:
-			if (!opt.quiet)
+				break;
+			case ELOOP:
 				weprintf("%s - too many levels of symbolic links - skipping", path);
-			break;
-		case EACCES:
-			if (!opt.quiet)
+				break;
+			case EACCES:
 				weprintf("you don't have permission to open %s - skipping", path);
-			break;
-		default:
-			if (!opt.quiet)
+				break;
+			case EOVERFLOW:
+				weprintf("Cannot open %s - EOVERFLOW.\n"
+					"Recompile with stat64=1 to fix this");
+				break;
+			default:
 				weprintf("couldn't open %s", path);
-			break;
+				break;
+			}
 		}
 		free(path);
 		return;
 	}
 
 	if ((S_ISDIR(st.st_mode)) && (level != FILELIST_LAST)) {
-		struct dirent *de;
+		struct dirent **de;
 		DIR *dir;
+		int cnt, n;
 
 		D(("It is a directory\n"));
 
@@ -186,13 +195,25 @@ void add_file_to_filelist_recursively(char *origpath, unsigned char level)
 			free(path);
 			return;
 		}
-		de = readdir(dir);
-		while (de != NULL) {
-			if (strcmp(de->d_name, ".")
-					&& strcmp(de->d_name, "..")) {
+		n = scandir(path, &de, file_selector_all, alphasort);
+		if (n < 0) {
+		  switch (errno) {
+		  case ENOMEM:
+		    if (!opt.quiet)
+		      weprintf("Insufficient memory to scan directory %s:", path);
+		    break;
+		  default:
+		    if (!opt.quiet)
+		      weprintf("Failed to scan directory %s:", path);
+		  }
+		}
+
+		for (cnt = 0; cnt < n; cnt++) {
+			if (strcmp(de[cnt]->d_name, ".")
+					&& strcmp(de[cnt]->d_name, "..")) {
 				char *newfile;
 
-				newfile = estrjoin("", path, "/", de->d_name, NULL);
+				newfile = estrjoin("", path, "/", de[cnt]->d_name, NULL);
 
 				/* This ensures we go down one level even if not fully recursive
 				   - this way "feh some_dir" expands to some_dir's contents */
@@ -203,8 +224,9 @@ void add_file_to_filelist_recursively(char *origpath, unsigned char level)
 
 				free(newfile);
 			}
-			de = readdir(dir);
+			free(de[cnt]);
 		}
+		free(de);
 		closedir(dir);
 	} else if (S_ISREG(st.st_mode)) {
 		D(("Adding regular file %s to filelist\n", path));
@@ -236,7 +258,7 @@ gib_list *feh_file_info_preload(gib_list * list)
 	gib_list *remove_list = NULL;
 
 	if (opt.verbose)
-		fprintf(stdout, PACKAGE " - preloading...\n");
+		fputs(PACKAGE " - preloading...\n", stdout);
 
 	for (l = list; l; l = l->next) {
 		file = FEH_FILE(l->data);
@@ -250,7 +272,7 @@ gib_list *feh_file_info_preload(gib_list * list)
 			feh_display_status('.');
 	}
 	if (opt.verbose)
-		fprintf(stdout, "\n");
+		feh_display_status(0);
 
 	if (remove_list) {
 		for (l = remove_list; l; l = l->next)
@@ -275,36 +297,33 @@ int feh_file_info_load(feh_file * file, Imlib_Image im)
 
 	errno = 0;
 	if (stat(file->filename, &st)) {
-		/* Display useful error message */
-		switch (errno) {
-		case ENOENT:
-		case ENOTDIR:
-			if (!opt.quiet)
+		if (!opt.quiet) {
+			switch (errno) {
+			case ENOENT:
+			case ENOTDIR:
 				weprintf("%s does not exist - skipping", file->filename);
-			break;
-		case ELOOP:
-			if (!opt.quiet)
+				break;
+			case ELOOP:
 				weprintf("%s - too many levels of symbolic links - skipping", file->filename);
-			break;
-		case EACCES:
-			if (!opt.quiet)
+				break;
+			case EACCES:
 				weprintf("you don't have permission to open %s - skipping", file->filename);
-			break;
-		default:
-			if (!opt.quiet)
+				break;
+			default:
 				weprintf("couldn't open %s ", file->filename);
-			break;
+				break;
+			}
 		}
 		return(1);
 	}
 
 	if (im)
 		im1 = im;
-	else if (!feh_load_image(&im1, file))
-		return(1);
-
-	if (!im1)
+	else if (!feh_load_image(&im1, file) || !im1) {
+		if (!opt.quiet)
+			weprintf("couldn't load %s", file->filename);
 		return(1);
+	}
 
 	file->info = feh_file_info_new();
 
@@ -319,7 +338,7 @@ int feh_file_info_load(feh_file * file, Imlib_Image im)
 
 	file->info->size = st.st_size;
 
-	if (need_free && im1)
+	if (need_free)
 		gib_imlib_free_image_and_decache(im1);
 	return(0);
 }
@@ -419,7 +438,7 @@ int feh_write_filelist(gib_list * list, char *filename)
 	FILE *fp;
 	gib_list *l;
 
-	if (!list || !filename)
+	if (!list || !filename || !strcmp(filename, "/dev/stdin"))
 		return(0);
 
 	errno = 0;
@@ -441,24 +460,10 @@ gib_list *feh_read_filelist(char *filename)
 	FILE *fp;
 	gib_list *list = NULL;
 	char s[1024], s1[1024];
-	Imlib_Image im1;
 
 	if (!filename)
 		return(NULL);
 
-	/* try and load the given filelist as an image, cowardly refuse to
-	 * overwrite an image with a filelist. (requested by user who did feh -df *
-	 * when he meant feh -dF *, as it overwrote the first image with the
-	 * filelist).
-	 */
-	if (feh_load_image_char(&im1, filename)) {
-		weprintf(
-				"The file you specified as a filelist to read - %s - appears to be an image. Ignoring it (this is a common mistake).\n",
-				filename);
-		opt.filelistfile = NULL;
-		return(NULL);
-	}
-
 	errno = 0;
 	if ((fp = fopen(filename, "r")) == NULL) {
 		/* return quietly, as it's okay to specify a filelist file that doesn't
