@@ -390,6 +390,7 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		{"thumb-redraw"  , 1, 0, 'J'},
 		{"info"          , 1, 0, 234},
 		{"force-aliasing", 0, 0, 235},
+		{"no-fehbg"      , 0, 0, 236},
 
 		{0, 0, 0, 0}
 	};
@@ -703,6 +704,9 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		case 235:
 			opt.force_aliasing = 1;
 			break;
+		case 236:
+			opt.no_fehbg = 1;
+			break;
 		default:
 			break;
 		}
