@@ -390,7 +390,7 @@ char *feh_printf(char *str, feh_file * file)
 	ret[0] = '\0';
 
 	for (c = str; *c != '\0'; c++) {
-		if (*c == '%') {
+		if ((*c == '%') && (*(c+1) != '\0')) {
 			c++;
 			switch (*c) {
 			case 'f':
@@ -402,41 +402,38 @@ char *feh_printf(char *str, feh_file * file)
 					strcat(ret, file->name);
 				break;
 			case 'w':
-				if (file) {
-					if (!file->info)
-						feh_file_info_load(file, NULL);
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
 					snprintf(buf, sizeof(buf), "%d", file->info->width);
 					strcat(ret, buf);
 				}
 				break;
 			case 'h':
-				if (file) {
-					if (!file->info)
-						feh_file_info_load(file, NULL);
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
 					snprintf(buf, sizeof(buf), "%d", file->info->height);
 					strcat(ret, buf);
 				}
 				break;
 			case 's':
-				if (file) {
-					if (!file->info)
-						feh_file_info_load(file, NULL);
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
 					snprintf(buf, sizeof(buf), "%d", file->info->size);
 					strcat(ret, buf);
 				}
 				break;
+			case 'S':
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
+					snprintf(buf, sizeof(buf),
+							"%.2fkB", ((double)file->info->size / 1000));
+					strcat(ret, buf);
+				}
+				break;
 			case 'p':
-				if (file) {
-					if (!file->info)
-						feh_file_info_load(file, NULL);
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
 					snprintf(buf, sizeof(buf), "%d", file->info->pixels);
 					strcat(ret, buf);
 				}
 				break;
 			case 't':
-				if (file) {
-					if (!file->info)
-						feh_file_info_load(file, NULL);
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
 					strcat(ret, file->info->format);
 				}
 				break;
@@ -459,18 +456,22 @@ char *feh_printf(char *str, feh_file * file)
 					 + 1 : 0);
 				strcat(ret, buf);
 				break;
+			case '%':
+				strcat(ret, "%");
+				break;
 			default:
-				strncat(ret, c, 1);
+				weprintf("Unrecognized format specifier %%%c", *c);
+				strncat(ret, c - 1, 2);
 				break;
 			}
-		} else if (*c == '\\') {
+		} else if ((*c == '\\') && (*(c+1) != '\0')) {
 			c++;
 			switch (*c) {
 			case 'n':
 				strcat(ret, "\n");
 				break;
 			default:
-				strncat(ret, c, 1);
+				strncat(ret, c - 1, 2);
 				break;
 			}
 		} else
