@@ -381,6 +381,7 @@ void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 	int sx, sy, sw, sh, dx, dy, dw, dh;
 	int calc_w, calc_h;
 	int antialias = 0;
+	int need_center = winwid->had_resize;
 
 	if (!winwid->full_screen && resize) {
 		winwidget_resize(winwid, winwid->im_w, winwid->im_h);
@@ -415,6 +416,12 @@ void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 				     || (winwid->has_rotated)))
 		feh_draw_checks(winwid);
 
+	if (!winwid->full_screen && opt.zoom_mode
+				     && (winwid->zoom == 1.0) && !opt.geom_flags
+				     && (winwid->w > winwid->im_w) && (winwid->h > winwid->im_h))
+		feh_calc_needed_zoom(&(winwid->zoom), winwid->im_w, winwid->im_h, winwid->w, winwid->h);
+
+
 	if (resize && (winwid->full_screen || opt.geom_flags)) {
 		int smaller;	/* Is the image smaller than screen? */
 		int max_w = 0, max_h = 0;
@@ -503,6 +510,10 @@ void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 			winwid->im_y = (int) (max_h - (winwid->im_h * winwid->zoom)) >> 1;
 		}
 	}
+	else if (need_center && !winwid->full_screen && opt.scale_down) {
+		winwid->im_x = (int) (winwid->w - (winwid->im_w * winwid->zoom)) >> 1;
+		winwid->im_y = (int) (winwid->h - (winwid->im_h * winwid->zoom)) >> 1;
+	}
 
 	/* Now we ensure only to render the area we're looking at */
 	dx = winwid->im_x;
